#=============================================================================
# Copyright (c) 2018-2021, NVIDIA CORPORATION.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==========================================================================

#[=======================================================================[

nvCOMP is a CUDA library that features generic compression interfaces to enable
developers to use high-performance GPU compressors and decompressors in their applications.

Result Variables
^^^^^^^^^^^^^^^^

This module will construct the following targets::
  nvcomp::nvcomp

This module will set the following variables::

  NVCOMP_FOUND

#]=======================================================================]


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was nvcomp-config.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

find_package(CUDAToolkit REQUIRED)

include("${CMAKE_CURRENT_LIST_DIR}/nvcomp-targets-common.cmake")
if (EXISTS "${CMAKE_CURRENT_LIST_DIR}/nvcomp-targets-static.cmake")
	include("${CMAKE_CURRENT_LIST_DIR}/nvcomp-targets-static.cmake")
endif()
if (EXISTS "${CMAKE_CURRENT_LIST_DIR}/nvcomp-targets-dynamic.cmake")
	include("${CMAKE_CURRENT_LIST_DIR}/nvcomp-targets-dynamic.cmake")
endif()

check_required_components(nvcomp)

set(${CMAKE_FIND_PACKAGE_NAME}_CONFIG "${CMAKE_CURRENT_LIST_FILE}")

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(${CMAKE_FIND_PACKAGE_NAME} CONFIG_MODE)
